@@ -36,11 +36,10 @@
 // Local Includes
 #include "tabbar.h"
 #include "application.h"
-#include "mainwindow.h"
 #include "history.h"
-#include "stackedurlbar.h"
 #include "urlbar.h"
 #include "webview.h"
+#include "sessionmanager.h"
 
 // KDE Includes
 #include <KUrl>
@@ -50,6 +49,7 @@
 #include <KMessageBox>
 #include <KDebug>
 #include <KStandardDirs>
+#include <KPassivePopup>
 
 // Qt Includes
 #include <QtCore/QTimer>
@@ -60,35 +60,40 @@
 #include <QtGui/QLabel>
 #include <QtGui/QMovie>
 #include <QtGui/QWidget>
-#include <QtGui/QMouseEvent>
 
 
-MainView::MainView(QWidget *parent)
+MainView::MainView(MainWindow *parent)
         : KTabWidget(parent)
-        , m_urlBars(new StackedUrlBar(this))
+        , m_urlBar(new UrlBar(this))
         , m_tabBar(new TabBar(this))
         , m_addTabButton(new QToolButton(this))
+        , m_currentTabIndex(0)
+        , m_parentWindow(parent)
 {
     // setting tabbar
     setTabBar(m_tabBar);
 
+    // set mouse tracking for tab previews
+    setMouseTracking(true);
+     
     // loading pixmap path
     m_loadingGitPath = KStandardDirs::locate("appdata" , "pics/loading.mng");
 
     // connecting tabbar signals
-    connect(m_tabBar, SIGNAL(closeTab(int)), this, SLOT(slotCloseTab(int)));
-    connect(m_tabBar, SIGNAL(cloneTab(int)), this, SLOT(slotCloneTab(int)));
-    connect(m_tabBar, SIGNAL(closeOtherTabs(int)), this, SLOT(slotCloseOtherTabs(int)));
-    connect(m_tabBar, SIGNAL(reloadTab(int)), this, SLOT(slotReloadTab(int)));
-    connect(m_tabBar, SIGNAL(reloadAllTabs()), this, SLOT(slotReloadAllTabs()));
-    connect(m_tabBar, SIGNAL(tabMoved(int, int)), this, SLOT(moveTab(int, int)));
+    connect(m_tabBar, SIGNAL(closeTab(int)), this, SLOT(closeTab(int)));
+    connect(m_tabBar, SIGNAL(mouseMiddleClick(int)), this, SLOT(closeTab(int)));
+    connect(m_tabBar, SIGNAL(newTabRequest()), this, SLOT(newTab()));
+    
+    connect(m_tabBar, SIGNAL(cloneTab(int)), this, SLOT(cloneTab(int)));
+    connect(m_tabBar, SIGNAL(closeOtherTabs(int)), this, SLOT(closeOtherTabs(int)));
+    connect(m_tabBar, SIGNAL(reloadTab(int)), this, SLOT(reloadTab(int)));
+    connect(m_tabBar, SIGNAL(reloadAllTabs()), this, SLOT(reloadAllTabs()));
 
+    connect(m_tabBar, SIGNAL(tabCloseRequested(int)), this, SLOT(closeTab(int)));
+    
     // current page index changing
-    connect(this, SIGNAL(currentChanged(int)), this, SLOT(slotCurrentChanged(int)));
-
-    setTabsClosable(true);
-    connect(m_tabBar, SIGNAL(tabCloseRequested(int)), this, SLOT(slotCloseTab(int)));
-
+    connect(this, SIGNAL(currentChanged(int)), this, SLOT(currentChanged(int)));
+    
     QTimer::singleShot(0, this, SLOT(postLaunch()));
 }
 
@@ -100,18 +105,24 @@ MainView::~MainView()
 
 void MainView::postLaunch()
 {
-    m_addTabButton->setDefaultAction(Application::instance()->mainWindow()->actionByName("new_tab"));
+    // Session Manager
+    connect (this, SIGNAL(tabsChanged()), Application::sessionManager(), SLOT(saveSession()));
+    
+    m_addTabButton->setDefaultAction(m_parentWindow->actionByName("new_tab"));
+
     m_addTabButton->setAutoRaise(true);
     m_addTabButton->setToolButtonStyle(Qt::ToolButtonIconOnly);
 }
 
 
-void MainView::addTabButtonPosition()
+void MainView::updateTabButtonPosition()
 {
+    kDebug() << "updating new tab button position..";
+    
     static bool ButtonInCorner = false;
 
     int tabWidgetWidth = frameSize().width();
-    int tabBarWidth = tabBar()->tabSizeHint(0).width()*tabBar()->count();
+    int tabBarWidth = m_tabBar->tabSizeHint(0).width()*m_tabBar->count();
 
     if (tabBarWidth + m_addTabButton->width() > tabWidgetWidth)
     {
@@ -129,36 +140,39 @@ void MainView::addTabButtonPosition()
             ButtonInCorner = false;
         }
 
-        int newPos = tabWidgetWidth - m_addTabButton->width();
-        m_addTabButton->move(newPos, 0);
+        // detecting X position
+        int newPosX = tabBarWidth;      
+        int tabWidthHint = m_tabBar->tabSizeHint(0).width();
+        if (tabWidthHint < sizeHint().width()/4)
+            newPosX = tabWidgetWidth - m_addTabButton->width();
 
-        if (tabBar()->tabSizeHint(0).width()>=sizeHint().width()/4)
-            m_addTabButton->move(tabBarWidth, 0);
-        else
-            m_addTabButton->move(tabWidgetWidth - m_addTabButton->width(), 0);
+        // detecting Y position
+        int newPosY = m_tabBar->height() - m_addTabButton->height();
+        if(newPosY < 0)
+            newPosY = 5;    // this hardcoded value is used in just ONE situation:
+                            // the first time an user changes the "Always Show Tab Bar" settings
+                            // try some better fixes, if you can :D
+
+        m_addTabButton->move(newPosX, newPosY);
     }
 }
 
 
-UrlBar *MainView::currentUrlBar() const 
-{ 
-    return urlBar(-1); 
-}
-
-
-TabBar *MainView::tabBar() const 
-{ 
-    return m_tabBar; 
-}
-
 QToolButton *MainView::addTabButton() const
 {
     return m_addTabButton;
 }
 
-StackedUrlBar *MainView::urlBarStack() const 
+
+TabBar *MainView::tabBar() const
+{
+    return m_tabBar; 
+}
+
+
+UrlBar *MainView::urlBar() const 
 { 
-    return m_urlBars; 
+    return m_urlBar; 
 }
 
 
@@ -174,7 +188,7 @@ int MainView::webViewIndex(WebView *webView) const
 }
 
 
-void MainView::showTabBar()
+void MainView::updateTabBar()
 {
     if (ReKonfig::alwaysShowTabBar())
     {
@@ -183,6 +197,7 @@ void MainView::showTabBar()
             m_tabBar->show();
             m_addTabButton->show();
         }
+        updateTabButtonPosition();
         return;
     }
 
@@ -196,13 +211,14 @@ void MainView::showTabBar()
         if (m_tabBar->isHidden())
         {
             m_tabBar->show();
-            m_addTabButton->show(); 
+            m_addTabButton->show();
         }
+        updateTabButtonPosition();
     }
 }
 
 
-void MainView::slotWebReload()
+void MainView::webReload()
 {
     WebView *webView = currentWebView();
     QAction *action = webView->page()->action(QWebPage::Reload);
@@ -210,7 +226,7 @@ void MainView::slotWebReload()
 }
 
 
-void MainView::slotWebStop()
+void MainView::webStop()
 {
     WebView *webView = currentWebView();
     QAction *action = webView->page()->action(QWebPage::Stop);
@@ -218,76 +234,19 @@ void MainView::slotWebStop()
 }
 
 
-void MainView::slotWebBack()
-{
-    WebView *webView = currentWebView();
-    QAction *action = webView->page()->action(QWebPage::Back);
-    action->trigger();
-}
-
-
-void MainView::slotWebForward()
-{
-    WebView *webView = currentWebView();
-    QAction *action = webView->page()->action(QWebPage::Forward);
-    action->trigger();
-}
-
-
-void MainView::slotWebUndo()
-{
-    WebView *webView = currentWebView();
-    QAction *action = webView->page()->action(QWebPage::Undo);
-    action->trigger();
-}
-
-
-void MainView::slotWebRedo()
-{
-    WebView *webView = currentWebView();
-    QAction *action = webView->page()->action(QWebPage::Redo);
-    action->trigger();
-}
-
-
-void MainView::slotWebCut()
-{
-    WebView *webView = currentWebView();
-    QAction *action = webView->page()->action(QWebPage::Cut);
-    action->trigger();
-}
-
-
-void MainView::slotWebCopy()
-{
-    WebView *webView = currentWebView();
-    QAction *action = webView->page()->action(QWebPage::Copy);
-    action->trigger();
-}
-
-
-void MainView::slotWebPaste()
-{
-    WebView *webView = currentWebView();
-    QAction *action = webView->page()->action(QWebPage::Paste);
-    action->trigger();
-}
-
-
 void MainView::clear()
 {
-    // clear the line edit history
-    for (int i = 0; i < m_urlBars->count(); ++i)
-    {
-        /// TODO What exactly do we need to clear here?
-        urlBar(i)->clearHistory();
-        urlBar(i)->clear();
-    }
+    // FIXME (the programmer, not the code)
+    // What exactly do we need to clear here?
+    m_urlBar->clearHistory();
+    m_urlBar->clear();
+
+    m_recentlyClosedTabs.clear();
 }
 
 
 // When index is -1 index chooses the current tab
-void MainView::slotReloadTab(int index)
+void MainView::reloadTab(int index)
 {
     if (index < 0)
         index = currentIndex();
@@ -300,35 +259,51 @@ void MainView::slotReloadTab(int index)
 }
 
 
-void MainView::slotCurrentChanged(int index)
+// this slot is called on tab switching
+void MainView::currentChanged(int index)
 {
+    // retrieve the webview related to the index
     WebView *webView = this->webView(index);
     if (!webView)
         return;
 
-    Q_ASSERT(m_urlBars->count() == count());
+    // retrieve the old webview (that where we move from)
+    WebView *oldWebView = this->webView(m_currentTabIndex);
+    
+    // set current index
+    m_currentTabIndex = index;
 
-    WebView *oldWebView = this->webView(m_urlBars->currentIndex());
     if (oldWebView)
-    {
+    {        
+        // disconnecting webview from urlbar
+        disconnect(oldWebView, SIGNAL(loadProgress(int)), urlBar(), SLOT(updateProgress(int)));
+        disconnect(oldWebView, SIGNAL(loadFinished(bool)), urlBar(), SLOT(loadFinished(bool)));
+        disconnect(oldWebView, SIGNAL(urlChanged(const QUrl &)), urlBar(), SLOT(setUrl(const QUrl &)));
+    
+        // disconnecting webpage from mainview
         disconnect(oldWebView->page(), SIGNAL(statusBarMessage(const QString&)),
                    this, SIGNAL(showStatusBarMessage(const QString&)));
         disconnect(oldWebView->page(), SIGNAL(linkHovered(const QString&, const QString&, const QString&)),
                    this, SIGNAL(linkHovered(const QString&)));
     }
 
+    // connecting webview with urlbar
+    connect(webView, SIGNAL(loadProgress(int)), urlBar(), SLOT(updateProgress(int)));
+    connect(webView, SIGNAL(loadFinished(bool)), urlBar(), SLOT(loadFinished(bool)));
+    connect(webView, SIGNAL(urlChanged(const QUrl &)), urlBar(), SLOT(setUrl(const QUrl &)));
+    
     connect(webView->page(), SIGNAL(statusBarMessage(const QString&)), 
             this, SIGNAL(showStatusBarMessage(const QString&)));
     connect(webView->page(), SIGNAL(linkHovered(const QString&, const QString&, const QString&)), 
             this, SIGNAL(linkHovered(const QString&)));
 
     emit setCurrentTitle(webView->title());
-    m_urlBars->setCurrentIndex(index);
-    currentUrlBar()->setUrl(webView->url());
+    urlBar()->setUrl(webView->url());
+    urlBar()->setProgress(webView->progress());
     emit showStatusBarMessage(webView->lastStatusBarText());
 
     // notify UI to eventually switch stop/reload button
-    if(currentUrlBar()->isLoading())
+    if(urlBar()->isLoading())
         emit browserTabLoading(true);
     else
         emit browserTabLoading(false);
@@ -338,22 +313,6 @@ void MainView::slotCurrentChanged(int index)
 }
 
 
-UrlBar *MainView::urlBar(int index) const
-{
-    if (index == -1)
-    {
-        index = m_urlBars->currentIndex();
-    }
-    UrlBar *urlBar = m_urlBars->urlBar(index);
-    if (urlBar)
-    {
-        return urlBar;
-    }
-    kWarning() << "URL bar with index" << index << "not found. Returning NULL. (line:" << __LINE__ << ")";
-    return NULL;
-}
-
-
 WebView *MainView::webView(int index) const
 {
     QWidget *widget = this->widget(index);
@@ -362,30 +321,15 @@ WebView *MainView::webView(int index) const
         return webView;
     }
 
-    kWarning() << "WebView with index " << index << "not found. Returning NULL." ;
+    kDebug() << "WebView with index " << index << "not found. Returning NULL." ;
     return 0;
 }
 
 
-// FIXME (0.3 target, after OneUrlBar). Divide in 2 functions:
-// 1. the slot void newTab() to create a "new empty focused tab"
-// 2. the public method WebView *newWebView() to just create a new webview 
-// without working with the focus and loading an url
-WebView *MainView::newTab(bool focused)
+WebView *MainView::newWebView(bool focused, bool nearParent)
 {
-    // line edit
-    UrlBar *urlBar = new UrlBar;  // Ownership of widget is passed on to the QStackedWidget (addWidget method).
-    connect(urlBar, SIGNAL(activated(const KUrl&)), Application::instance(), SLOT(loadUrl(const KUrl&)));
-    m_urlBars->addUrlBar(urlBar);
-
     WebView *webView = new WebView;  // should be deleted on tab close?
 
-    // connecting webview with urlbar
-    connect(webView, SIGNAL(loadProgress(int)), urlBar, SLOT(slotUpdateProgress(int)));
-    connect(webView, SIGNAL(loadFinished(bool)), urlBar, SLOT(slotLoadFinished(bool)));
-    connect(webView, SIGNAL(urlChanged(const QUrl &)), urlBar, SLOT(setUrl(const QUrl &)));
-    connect(webView, SIGNAL(iconChanged()), urlBar, SLOT(slotUpdateUrl()));
-
     // connecting webview with mainview
     connect(webView, SIGNAL(loadStarted()), this, SLOT(webViewLoadStarted()));
     connect(webView, SIGNAL(loadFinished(bool)), this, SLOT(webViewLoadFinished(bool)));
@@ -393,32 +337,51 @@ WebView *MainView::newTab(bool focused)
     connect(webView, SIGNAL(titleChanged(const QString &)), this, SLOT(webViewTitleChanged(const QString &)));
     connect(webView, SIGNAL(urlChanged(const QUrl &)), this, SLOT(webViewUrlChanged(const QUrl &)));
 
-    connect(webView, SIGNAL(ctrlTabPressed()), this, SLOT(nextTab()));
-    connect(webView, SIGNAL(shiftCtrlTabPressed()), this, SLOT(previousTab()));
-
     // connecting webPage signals with mainview
     connect(webView->page(), SIGNAL(windowCloseRequested()), this, SLOT(windowCloseRequested()));
     connect(webView->page(), SIGNAL(printRequested(QWebFrame *)), this, SIGNAL(printRequested(QWebFrame *)));
 
-    addTab(webView, i18n("(Untitled)"));
+    if (nearParent)
+        insertTab(currentIndex() + 1, webView, i18n("(Untitled)"));
+    else
+        addTab(webView, i18n("(Untitled)"));
+
+    updateTabBar();
     
     if (focused)
     {
         setCurrentWidget(webView);
     }
-    
-    urlBar->setFocus();
 
     emit tabsChanged();
-
-    showTabBar();
-    addTabButtonPosition();
-
+    
     return webView;
 }
 
 
-void MainView::slotReloadAllTabs()
+void MainView::newTab()
+{
+    WebView *w = newWebView();
+
+    switch(ReKonfig::newTabsBehaviour())
+    {
+    case 0: // new tab page
+        if(m_parentWindow->newTabPage())
+            break;
+    case 1: // blank page
+        urlBar()->setUrl(KUrl(""));
+        break;
+    case 2: // homepage
+        w->load( QUrl(ReKonfig::homePage()) );
+        break;
+    default:
+        break;
+    }
+    urlBar()->setFocus();
+}
+
+
+void MainView::reloadAllTabs()
 {
     for (int i = 0; i < count(); ++i)
     {
@@ -433,7 +396,6 @@ void MainView::slotReloadAllTabs()
 
 void MainView::windowCloseRequested()
 {
-
     WebPage *webPage = qobject_cast<WebPage*>(sender());
     WebView *webView = qobject_cast<WebView*>(webPage->view());
     int index = webViewIndex(webView);
@@ -442,57 +404,60 @@ void MainView::windowCloseRequested()
     {
         if (count() == 1)
         {
-            Application::instance()->mainWindow()->close();
+            m_parentWindow->close();
         }
         else
         {
-            slotCloseTab(index);
+            closeTab(index);
         }
+        return;
     }
-    else
-    {
-        kWarning() << "Invalid tab index" << "line:" << __LINE__;
-    }
+    kDebug() << "Invalid tab index" << "line:" << __LINE__;
 }
 
 
-void MainView::slotCloseOtherTabs(int index)
+void MainView::closeOtherTabs(int index)
 {
     if (-1 == index)
         return;
 
     for (int i = count() - 1; i > index; --i)
     {
-        slotCloseTab(i);
+        closeTab(i);
     }
 
     for (int i = index - 1; i >= 0; --i)
     {
-        slotCloseTab(i);
+        closeTab(i);
     }
 
-    showTabBar();
-    addTabButtonPosition();
+    updateTabBar();
 }
 
 
 // When index is -1 index chooses the current tab
-void MainView::slotCloneTab(int index)
+void MainView::cloneTab(int index)
 {
     if (index < 0)
         index = currentIndex();
     if (index < 0 || index >= count())
         return;
-    WebView *tab = newTab();
-    tab->setUrl(webView(index)->url());
+    
+    WebView *tab = newWebView();    
+    KUrl url = webView(index)->url();
+    
+    // workaround against bug in webkit:
+    // only set url if it is not empty
+    // otherwise the current working directory will be used
+    if (!url.isEmpty())
+        tab->setUrl(url);
 
-    showTabBar();
-    addTabButtonPosition();
+    updateTabBar();
 }
 
 
 // When index is -1 index chooses the current tab
-void MainView::slotCloseTab(int index)
+void MainView::closeTab(int index)
 {
     // do nothing if just one tab is opened
     if (count() == 1)
@@ -509,21 +474,29 @@ void MainView::slotCloseTab(int index)
         if (tab->isModified())
         {
             int risp = KMessageBox::questionYesNo(this,
-                        i18n("You have modified this page and when closing it you would lose the modifications.\n"
-                             "Do you really want to close this page?\n"),
-                        i18n("Do you really want to close this page?"));
+                        i18n("This tab contains changes that have not been submitted.\n"
+                             "Closing the tab will discard these changes.\n"
+                             "Do you really want to close this tab?\n"),
+                        i18n("Closing Modified Tab"));
             if (risp == KMessageBox::No)
                 return;
         }
         hasFocus = tab->hasFocus();
-    }
 
-    QWidget *urlBar = m_urlBars->urlBar(index);
-    m_urlBars->removeWidget(urlBar);
-    urlBar->deleteLater();   // urlBar is scheduled for deletion.
+        //store close tab except homepage
+        if (!tab->url().prettyUrl().startsWith( QLatin1String("about:") ) && !tab->url().isEmpty())
+        {
+            QString title = tab->title();
+            QString url = tab->url().prettyUrl();
+            HistoryItem item(url, QDateTime::currentDateTime(), title);
+            m_recentlyClosedTabs.removeAll(item);
+            m_recentlyClosedTabs.prepend(item);
+        }
+    }
 
     QWidget *webView = widget(index);
     removeTab(index);
+    updateTabBar();         // UI operation: do it ASAP!!
     webView->deleteLater();  // webView is scheduled for deletion.
 
     emit tabsChanged();
@@ -532,9 +505,6 @@ void MainView::slotCloseTab(int index)
     {
         currentWebView()->setFocus();
     }
-
-    showTabBar();
-    addTabButtonPosition();
 }
 
 
@@ -575,7 +545,7 @@ void MainView::webViewLoadFinished(bool ok)
 
     webViewIconChanged();
     emit browserTabLoading(false);
-    
+
     // don't display messages for background tabs
     if (index != currentIndex())
     {
@@ -601,6 +571,8 @@ void MainView::webViewIconChanged()
         delete movie;
         label->setMovie(0);
         label->setPixmap(icon.pixmap(16, 16));
+
+        urlBar()->updateUrl();
     }
 }
 
@@ -656,14 +628,6 @@ void MainView::previousTab()
 }
 
 
-void MainView::moveTab(int fromIndex, int toIndex)
-{
-    QWidget *lineEdit = m_urlBars->widget(fromIndex);
-    m_urlBars->removeWidget(lineEdit);
-    m_urlBars->insertWidget(toIndex, lineEdit);
-}
-
-
 QLabel *MainView::animatedLoading(int index, bool addMovie)
 {
     if (index == -1)
@@ -687,20 +651,14 @@ QLabel *MainView::animatedLoading(int index, bool addMovie)
 }
 
 
-void MainView::mouseDoubleClickEvent(QMouseEvent *event) //WARNING Need to be fix
+QList<HistoryItem> MainView::recentlyClosedTabs()
 {
-    if (!childAt(event->pos()))
-    {
-        newTab();
-        return;
-    }
-    KTabWidget::mouseDoubleClickEvent(event);
+    return m_recentlyClosedTabs;
 }
 
 
 void MainView::resizeEvent(QResizeEvent *event)
 {
-    addTabButtonPosition();
+    updateTabBar();
     KTabWidget::resizeEvent(event);
 }
-
