@@ -27,47 +27,21 @@
 #ifndef NETWORKACCESSMANAGER_H
 #define NETWORKACCESSMANAGER_H
 
-#include <kdeversion.h>
 
-#if KDE_IS_VERSION(4, 2, 70)
-    #include <kio/accessmanager.h>
-    using namespace KIO;
-    typedef AccessManager RekonqAccessManager;
-#else
-    #include <QtNetwork/QNetworkAccessManager>
-    typedef QNetworkAccessManager RekonqAccessManager;
-#endif
-
-// Forward Declarations
-class QNetworkDiskCache;
+// KDE Includes
+#include <kio/accessmanager.h>
 
 
-class NetworkAccessManager : public RekonqAccessManager
+using namespace KIO;
+
+
+class NetworkAccessManager : public AccessManager
 {
     Q_OBJECT
 
 public:
     NetworkAccessManager(QObject *parent = 0);
-    KIO::MetaData& metaData();    
     void resetDiskCache();
-    
-public slots:
-    void loadSettings();
-
-protected:
-    virtual QNetworkReply *createRequest(Operation op, const QNetworkRequest &req, QIODevice *outgoingData = 0);
-
-private slots:
-    void authenticationRequired(QNetworkReply *reply, QAuthenticator *auth);
-    void proxyAuthenticationRequired(const QNetworkProxy &proxy, QAuthenticator *auth);
-
-#ifndef QT_NO_OPENSSL
-    void slotSSLErrors(QNetworkReply *reply, const QList<QSslError> &error);
-#endif
-
-private:
-    QNetworkDiskCache *m_diskCache;
-    KIO::MetaData m_metaData;
 };
 
 #endif // NETWORKACCESSMANAGER_H
